﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Speech_to_text.Windows;
using Ozeki.Common;


namespace Speech_to_text
{
    public partial class PhoneMain : Form
    {
        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        PhoneLineState phoneLineInformation;
        IPhoneCall call;
        bool inComingCall;

        PhoneCallAudioReceiver CallReceiver;
        PhoneCallAudioSender CallSender;

        Speaker Speaker;
        Microphone Microphone;
        SpeechToText SpeechToText;

        MediaConnector Connector;

        public PhoneMain()
        {
            InitializeComponent();
        }

        void SpeechToText_WordsRecognized(object sender, VoIPEventArgs<IEnumerable<string>> e)
        {
            InvokeGUIThread(() => 
            {
                foreach (string word in e.Item)
                    MessageBox.Show(word, "Keyword", MessageBoxButtons.OK, MessageBoxIcon.Information);
            });
        }

        void SpeechToText_WordsHypotyzed(object sender, VoIPEventArgs<IEnumerable<string>> e)
        {
            InvokeGUIThread(() =>
            {
                foreach (string word in e.Item)
                    MessageBox.Show(word, "Keyword", MessageBoxButtons.OK, MessageBoxIcon.Information);
            });
        }

        private void PhoneMain_Load(object sender, EventArgs e)
        {
            Tssl_Ozeki.Text = string.Format("© Copyright {0}, Ozeki Informatics Ltd.", DateTime.Now.Year);

            InitializeSoftPhone();

            // set default data 
            Dgv_Words.Rows.Add("Welcome");
            Dgv_Words.Rows.Add("Hello");
        }


        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone line state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineInformation = e.Item;
            InvokeGUIThread(() =>
            {
                labelIdentifier.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                    labelRegStatus.Text = "Online";
                    labelCallStateInfo.Text = "Registration succeeded";
                }
                else
                    labelCallStateInfo.Text = e.Item.ToString();
            });
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
                         {
                             labelCallStateInfo.Text = "Incoming call";
                             labelDialingNumber.Text = String.Format("from {0}", e.Item.DialInfo);
                             call = e.Item;
                             WireUpCallEvents();
                             inComingCall = true;
                         });
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { labelCallStateInfo.Text = e.Item.ToString(); });

            switch (e.Item)
            {
                case CallState.InCall:
                    CallReceiver.AttachToCall(call);
                    CallSender.AttachToCall(call);
                    InitaializeAnalyze();
                    break;
                case CallState.Completed:
                    CallReceiver.Detach();
                    CallReceiver.Detach();
                    call = null;
                    InvokeGUIThread(() => { labelDialingNumber.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    call = null;
                    break;
            }
        }

        /// <summary>
        /// Initaialize a sound analysis.
        /// </summary>
        private void InitaializeAnalyze()
        {
            var words =
                (Dgv_Words.Rows.Cast<DataGridViewRow>().Where(e => e.Cells[0].Value != null).Select(
                    row => row.Cells[0].Value.ToString())).ToList();

            if (SpeechToText != null)
            {
                SpeechToText.WordsHypotyzed -= (SpeechToText_WordsHypotyzed);
                SpeechToText.WordsRecognized -= (SpeechToText_WordsRecognized);
                Connector.Disconnect(CallReceiver, SpeechToText);
                SpeechToText.Dispose();
            }

            SpeechToText = SpeechToText.CreateInstance(
                new AudioFormat(8000, 1, 16, 20), words.ToArray()
            );

            if (SpeechToText == null)
                return;

            SpeechToText.WordsHypotyzed += (SpeechToText_WordsHypotyzed);
            SpeechToText.WordsRecognized += (SpeechToText_WordsRecognized);

            Connector.Connect(CallReceiver, SpeechToText);
        }

        /// <summary>
        /// Displays DTMF signals.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_DtmfReceived(object sender, VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>> e)
        {
            InvokeGUIThread(() =>
                                {
                                    labelCallStateInfo.Text = String.Format("DTMF signal received: {0} ", e.Item.Item2.Signal);
                                });
        }

        /// <summary>
        /// There are certain situations when the call cannot be created, for example the dialed number is not available 
        /// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
        /// This event handling is for displaying these events.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
                         {
                             labelCallStateInfo.Text = e.Item.ToString();
                         });
        }

        #endregion

        #region Keypad events

        /// <summary>
        /// It starts a call with the dialed number or in case of an incoming call it accepts, picks up the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPickUp_Click(object sender, EventArgs e)
        {
            if (inComingCall)
            {
                inComingCall = false;
                call.Accept();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(labelDialingNumber.Text))
                return;

            if (phoneLineInformation != PhoneLineState.RegistrationSucceeded && phoneLineInformation != PhoneLineState.NoRegNeeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }

            call = softPhone.CreateCallObject(phoneLine, labelDialingNumber.Text);
            WireUpCallEvents();
            call.Start();
        }

        /// <summary>
        /// In the case of a call that is in progress it breaks the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                inComingCall = false;
                call.HangUp();
                call = null;
            }
            labelDialingNumber.Text = string.Empty;
        }

        /// <summary>
        /// It makes the dialing number.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_Click(object sender, EventArgs e)
        {
            var btn = sender as Button;
            if (btn != null)
            {
                if (btn.Text == "0/+")
                {
                    labelDialingNumber.Text += "0";
                }
                else
                    labelDialingNumber.Text += btn.Text.Trim();
            }
        }

        /// <summary>
        ///  It sends a DTMF signal according to the RFC 2833 standard.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_MouseDown(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StartDTMFSignal(VoIPMediaType.Audio, (DtmfNamedEvents)id);
                    }
                }
            }
        }
        /// <summary>
        /// It stops sending given DTMF signal. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPad_MouseUp(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StopDTMFSignal(VoIPMediaType.Audio, (DtmfNamedEvents)id);
                    }
                }
            }
        }

        #endregion

        #region Helper Functions
        /// <summary>
        ///It initializes a softphone object  with a SIP BPX, and it  is for requisiting a SIP account that is nedded  for a SIP PBX service. It registers this SIP
        ///account to the SIP PBX through an ’IphoneLine’ object  which represents the telephoneline. 
        ///If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);
                phoneLine = softPhone.CreatePhoneLine(new SIPAccount(true, "oz875", "oz875", "oz875", "oz875", "192.168.112.100", 5060),new NatConfiguration(NatTraversalMethod.None));
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);

                CallReceiver = new PhoneCallAudioReceiver();
                CallSender = new PhoneCallAudioSender();
                Speaker = Speaker.GetDefaultDevice();
                Microphone = Microphone.GetDefaultDevice();

                Connector = new MediaConnector();
                Connector.Connect(CallReceiver, Speaker);
                Connector.Connect(Microphone, CallSender);
                Speaker.Start();
                Microphone.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);
				
                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if(ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());				
            }
        }

        /// <summary>
        ///  It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents()
        {
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.DtmfReceived += new EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>>>(call_DtmfReceived);
            call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }
        #endregion

        #region Menu events
        private void Tsmi_HelpPage_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void Tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();

            aboutBox.ShowDialog(this);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion

        private void PhoneMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if(CallReceiver != null)
                CallReceiver.Dispose();

            if(CallSender != null)
                CallSender.Dispose();

            if(Speaker != null)
                Speaker.Dispose();

            if(Microphone != null)
                Microphone.Dispose();

            if(SpeechToText != null)
                SpeechToText.Dispose();

            if(Connector != null)
                Connector.Dispose();

            softPhone.Close();
        }

    }
}
